import {iTenderStatus} from "./iTenderStatus";
import Container from "./database/Container";
import {HCSR04} from "hc-sr04";
import {IContainer} from "./database/IContainer";
import Drink from "./database/Drink";
import {IDrink} from "./database/IDrink";
import debug from "debug";
import {WebSocketHandler} from "./WebSocketHandler";
import {IJob} from "./database/IJob";

const log = debug("itender:station");

export class iTender {
    static get containers(): { container: IContainer; sensor: HCSR04; pump: null }[] {
        return this._containers;
    }

    static get drinks(): IDrink[] {
        return this._drinks;
    }

    static get currentJob(): IJob | null {
        return this._currentJob;
    }

    private static _status: iTenderStatus = iTenderStatus.STARTING;
    private static _currentJob: IJob | null = null;

    private static _containers: { container: IContainer, sensor: HCSR04, pump: null }[] = [];
    private static _drinks: IDrink[];

    static setStatus(status: iTenderStatus) {
        this._status = status;
        if (WebSocketHandler.ws && WebSocketHandler.ws.readyState == 1)
            WebSocketHandler.sendStatus().then().catch(console.error);
    }

    static get status(): iTenderStatus {
        return this._status;
    }

    static startFill() {
        // todo Fill method


        // End
        this.measureContainers().then();
    }

    static cancelFill() {
        // todo Stop fill method
    }

    static measureContainers(): Promise<void> {
        log("Measuring containers...");

        return new Promise(async resolve => {
            for (let c of this._containers) {
                try {
                    let dist = c.sensor.distance();
                    c.container.filled = dist * 100 / (c.container.sensorFilledMax + c.container.sensorFilledMin);
                } catch (e) {
                    c.container.filled = -1;
                }
                await c.container.save();
            }
            log("Containers measured!");
            resolve();
        });
    }

    static refreshDrinks(): Promise<void> {
        this.setStatus(iTenderStatus.REFRESHING);
        log("Refreshing drinks...");
        return new Promise(async resolve => {
            this._drinks = [];
            for (let d of (await Drink.find().populate("ingredients.type"))) {

                let drinkAccept = true;
                for (let i of d.ingredients) {
                    let c = await Container.findOne({content: i["type"]});

                    if (!c) {
                        drinkAccept = false;
                        break;
                    }
                }
                if (drinkAccept) {
                    this._drinks.push(d);
                }

            }
            log("Drinks refreshed!");

            resolve();
            this.setStatus(iTenderStatus.READY);
        });
    }

    static refreshContainers(): Promise<void> {
        log("Refreshing containers...");
        this.setStatus(iTenderStatus.CALCULATING);
        return new Promise(async resolve => {
            let containers = await Container.find();
            for (let c of containers) {
                let i = 0;
                let found = false;
                for (let c2 of this._containers) {
                    if (c2.container._id == c._id) {
                        let sensor;
                        try {
                            sensor = new HCSR04(c.sensorTrigger, c.sensorEcho);
                        } catch (e) {
                        }
                        this._containers[i] = {
                            container: c,
                            sensor: sensor,
                            pump: null
                        };
                        found = true;
                        break;
                    }
                    i++;
                }
                if (!found) {
                    let sensor;
                    try {
                        sensor = new HCSR04(c.sensorTrigger, c.sensorEcho);
                    } catch (e) {
                    }
                    this._containers.push({
                        container: c,
                        sensor: sensor,
                        pump: null
                    });
                }

            }

            log("Containers refreshed!");
            this.measureContainers().then().catch(console.error);

            resolve();
            this.setStatus(iTenderStatus.READY);
        });
    }

    public static async autoCheckup() {
        setInterval(async () => {
            if (!this._currentJob)
                return;
            // Check if         startedTime     plus   2 mins   smaller than    now
            if (this._currentJob.startedAt.getTime() + 1000 * 60 * 2 <= Date.now()) {
                // Job can be declared as stuck!
                this._currentJob.successful = false;
                this._currentJob.endAt = new Date();
                await this._currentJob.save();
                this._currentJob = null;
            }
        }, 30000);
    }


}